
#include <istream>
#include <ostream>


namespace MFound {

  /* Encapsulates a generic linear solver.

     The interface is described below.

     A few high level comments:

     * There are typically two types of solvers: solvers based on
     * factorization (LU or CHoleski) and solvers that are iterative. In both
     * cases the solvers have some other parameters to be set. For iterative
     * solvers is the preconditioner,  precision and the maximum number of
     * iterations and for factorization solver is factorization method and
     * permutation. To keep thins simple we did not add these, so you can make
     * some decisions on this just document them in yoru readme file

     Another note: for us to test that the code compiles we put empty
     implementations. For your solution pls add the implementations in a
     different file called LSolver.inl and include this at the end of your
     header file

     The linear system is represented internally as one or multiple sparse
     matrices. Your solutiopn should use the sparse matrix implementation from
     the homework and not the one native to the solvers (TAUCS or Eigen).

     */

  template<class T>
  class GenericLSolver{
  public:

    // algebraic vector storage
    typedef typename T::Vector Vector;

    typedef typename T::Scalar Scalar;

    

    // default constructor
    GenericLSolver(){}

    /* 
       Creates a generic solver
       m - number of constraints (rows)
       n - number of variables 

       if m==n and is_LS is false this solve the system Ax=b
       if m>n or is_LS is true this solves teh system At*A = At*b;
       if m<n behaviour is unspecified
     */
    GenericLSolver(int m, int n, bool is_LS = false){}

    // query the size of the solver
    int& getM(){}
    int getM()const{}
    int& getN(){}
    int getN()const{}

    // API
    /* 
       The standard API for constructing a system is usually row by row as
       illustrated schematically below.This makes the editing of the sparse
       matrix more efficient. For instance (HINT) if we need to compute At * A
       it could be more efficient to do it as we go along rather than at the end.

       begin_system();
       
       begin_row();
       
       add(...
       ...
       add(...

       end_row()

       end_system();
     */
    // begin constructing the system
    virtual void begin_system(){}
    virtual void begin_row(){}

    /* adds an element to the linear system matrix
       However, the behaviour is different than in SparseMatrix.
       In SparseMatrix add(i,j) was M(i,j)+=c.
       Here sets M(i,j)=c

       However, elements on each rows can be specified in any order, not
       necessarily in the increasing order of column indices
    */
    virtual void add(int c, Scalar value){}

    virtual void end_row(){}


    /* If possible, the factorization of the matrices (which is the operation
    that takes the most time) should be done here (in end_system)

    The solve function should be fast, just do back substitution
    */
    virtual void end_system(){}
    
    /* 
       Solves for the b vector and puts the solution x in solution

       The solve function should be fast, just do back substitution so if the 
       implementation of the solver is based on a prefactorization, the
       factorization should ot be done here

    */
    virtual bool solve(Vector& solution, const Vector& b){}



  };// class




template<class T>
std::istream& operator>>(std::istream& f, GenericLSolver<T>&){}

template<class T>
std::ostream& operator<<(std::ostream& f, const GenericLSolver<T>&){}


}; // namespace
